"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _glob = _interopRequireDefault(require("glob"));

var _jayson = _interopRequireDefault(require("jayson"));

var _path = _interopRequireDefault(require("path"));

var _sane = _interopRequireDefault(require("sane"));

var _LintRunner = _interopRequireDefault(require("./LintRunner"));

var _eslint = require("eslint");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ROOT_DIR = process.cwd();
const eslint = new _eslint.ESLint({
  cwd: ROOT_DIR
});

class Server {
  constructor(options) {
    const {
      workers,
      port,
      paths,
      ignored,
      rcPath,
      quiet,
      fix
    } = options;
    this.port = port;
    this.rcPath = rcPath;
    this.cache = {};
    this.filesToProcess = 0;
    this.initializing = true;
    this.lintRunner = new _LintRunner.default(workers, !!quiet, fix === "true");

    const rootDir = _path.default.dirname(this.rcPath);

    this._setupWatcher(rootDir, paths.split(","), ignored.split(","));

    const that = this;

    const server = _jayson.default.server({
      status: function (args, cb) {
        if (that.initializing) {
          cb(null, {
            message: `Initializing`
          });
        } else if (that.filesToProcess === 0) {
          cb(null, that.getResultsFromCache());
        } else {
          const {
            errorCount,
            warningCount
          } = that.getResultsFromCache();
          cb(null, {
            message: `Linting...${that.filesToProcess} left to lint. Errors: ${errorCount} / Warnings: ${warningCount}`
          });
        }
      }
    });

    process.send({
      server: server
    });
    server.http().listen(this.port);
  }

  _setupWatcher(root, paths, ignored) {
    const watcher = (0, _sane.default)(root, {
      glob: paths,
      ignored: ignored,
      dot: true,
      watchman: false
    });
    watcher.on("ready", async () => {
      this.initializing = false;
      process.send({
        message: "Reading files to be linted...[this may take a little bit]"
      });
      let filePaths = [];

      for (let i = 0; i < paths.length; i++) {
        const files = _glob.default.sync(paths[i], {
          cwd: root,
          absolute: true,
          ignore: ignored
        });

        files.forEach(file => {
          filePaths.push(file);
        });
      }

      await this.lintAllFiles(filePaths);
    });
    watcher.on("change", async filepath => {
      let filePaths = [];

      if (filepath.indexOf(".eslintrc") !== -1) {
        this.cache = {};

        for (let i = 0; i < paths.length; i++) {
          const files = _glob.default.sync(paths[i], {
            cwd: root,
            absolute: true,
            ignore: ignored
          });

          files.forEach(file => {
            filePaths.push(file);
          });
        }

        await this.lintAllFiles(filePaths);
      } else {
        await this.lintFile(filepath);
      }
    });
    watcher.on("add", async filepath => {
      await this.lintFile(filepath);
    });
    watcher.on("delete", filepath => {
      delete this.cache[filepath];
    });
  }

  getResultsFromCache() {
    const records = Object.keys(this.cache).filter(filepath => {
      return this.cache[filepath] && (this.cache[filepath].errorCount > 0 || this.cache[filepath].warningCount > 0);
    }).map(filepath => this.cache[filepath]);
    const {
      errorCount,
      warningCount
    } = records.reduce((a, b) => ({
      errorCount: a.errorCount + b.errorCount,
      warningCount: a.warningCount + b.warningCount
    }), {
      errorCount: 0,
      warningCount: 0
    });
    return {
      records,
      errorCount,
      warningCount
    };
  }

  async lintFile(file) {
    if ((await eslint.isPathIgnored(file)) || file.indexOf("eslint") !== -1) {
      return;
    }

    this.filesToProcess++;
    const that = this;

    try {
      const results = await this.lintRunner.run([file]);
      const record = results.records[0];

      if (record) {
        delete record.source;
        that.cache[record.filePath] = record;
      }

      that.filesToProcess--;
    } catch (e) {
      console.error(e);
    }
  }

  async lintAllFiles(files) {
    await Promise.all(files.map(async file => {
      await this.lintFile(file);
    }));
  }

}

exports.default = Server;